// Matrix.c

#include "math.h"
#include "Matrix.h"
#include <string.h>

#define PI	3.14159268

void Matrix_Set(float * this, const float * pMatrix )
{
	memcpy( this, pMatrix, sizeof( float )*16 );
}

void Matrix_LoadIdentity(float * this)
{
	memset( this, 0, sizeof( float )*16 );
	this[0] = this[5] = this[10] = this[15] = 1.0f;
}

void Matrix_PostMultiply(float * this, const float * matrix )
{
	float newMatrix[16];

	newMatrix[0] = this[0] * matrix[0] + this[4] * matrix[1] + this[8] * matrix[2];
	newMatrix[1] = this[1] * matrix[0] + this[5] * matrix[1] + this[9] * matrix[2];
	newMatrix[2] = this[2] * matrix[0] + this[6] * matrix[1] + this[10] * matrix[2];
	newMatrix[3] = 0;

	newMatrix[4] = this[0] * matrix[4] + this[4] * matrix[5] + this[8] * matrix[6];
	newMatrix[5] = this[1] * matrix[4] + this[5] * matrix[5] + this[9] * matrix[6];
	newMatrix[6] = this[2] * matrix[4] + this[6] * matrix[5] + this[10] * matrix[6];
	newMatrix[7] = 0;

	newMatrix[8] = this[0] * matrix[8] + this[4] * matrix[9] + this[8] * matrix[10];
	newMatrix[9] = this[1] * matrix[8] + this[5] * matrix[9] + this[9] * matrix[10];
	newMatrix[10] = this[2] * matrix[8] + this[6] * matrix[9] + this[10] * matrix[10];
	newMatrix[11] = 0;

	newMatrix[12] = this[0] * matrix[12] + this[4] * matrix[13] + this[8] * matrix[14] + this[12];
	newMatrix[13] = this[1] * matrix[12] + this[5] * matrix[13] + this[9] * matrix[14] + this[13];
	newMatrix[14] = this[2] * matrix[12] + this[6] * matrix[13] + this[10] * matrix[14] + this[14];
	newMatrix[15] = 1;

	Matrix_Set(this, newMatrix );
}

void Matrix_SetTranslation(float * this, const float *translation )
{
	this[12] = translation[0];
	this[13] = translation[1];
	this[14] = translation[2];
}

void Matrix_SetInverseTranslation(float * this, const float *translation )
{
	this[12] = -translation[0];
	this[13] = -translation[1];
	this[14] = -translation[2];
}

void Matrix_SetRotationDegrees(float * this, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	Matrix_SetRotationRadians(this, vec );
}

void Matrix_SetInverseRotationRadians(float * this, const float * angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	this[0] = ( float )( cp*cy );
	this[4] = ( float )( cp*sy );
	this[8] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	this[1] = ( float )( srsp*cy-cr*sy );
	this[5] = ( float )( srsp*sy+cr*cy );
	this[9] = ( float )( sr*cp );

	this[2] = ( float )( crsp*cy+sr*sy );
	this[6] = ( float )( crsp*sy-sr*cy );
	this[10] = ( float )( cr*cp );
}

void Matrix_SetInverseRotationDegrees(float * this, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	Matrix_SetInverseRotationRadians( this, vec );
}

void Matrix_SetRotationRadians(float * this, const float *angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	this[0] = ( float )( cp*cy );
	this[1] = ( float )( cp*sy );
	this[2] = ( float )( -sp );
	this[3] = ( float )( 0.0f );

	double srsp = sr*sp;
	double crsp = cr*sp;

	this[4] = ( float )( srsp*cy-cr*sy );
	this[5] = ( float )( srsp*sy+cr*cy );
	this[6] = ( float )( sr*cp );

	this[8] = ( float )( crsp*cy+sr*sy );
	this[9] = ( float )( crsp*sy-sr*cy );
	this[10] = ( float )( cr*cp );
}

void Matrix_RotateVect(float * this, float * vec )
{
	float tmpvec[3];

	tmpvec[0] = vec[0] * this[0] + vec[1] * this[4] + vec[2] * this[8];
	tmpvec[1] = vec[0] * this[1] + vec[1] * this[5] + vec[2] * this[9];
	tmpvec[2] = vec[0] * this[2] + vec[1] * this[6] + vec[2] * this[10];

	memcpy( vec, tmpvec, sizeof( float )*3 );
}

void Matrix_InverseRotateVect(float * this, float *vec )
{
	float tmpvec[3];

	tmpvec[0] = vec[0] * this[0] + vec[1] * this[1] + vec[2] * this[2];
	tmpvec[1] = vec[0] * this[4] + vec[1] * this[5] + vec[2] * this[6];
	tmpvec[2] = vec[0] * this[8] + vec[1] * this[9] + vec[2] * this[10];

	memcpy( vec, tmpvec, sizeof( float )*3 );
}

void Matrix_TranslateVect(float * this, float *vec )
{
	vec[0] += this[12];
	vec[1] += this[13];
	vec[2] += this[14];
}

void Matrix_InverseTranslateVect(float * this, float *vec )
{
	vec[0] -= this[12];
	vec[1] -= this[13];
	vec[2] -= this[14];
}


