// MilkshapeModel.h

#ifndef _MILKSHAPEMODEL_H_
#define _MILKSHAPEMODEL_H_

#include "Model.h"

// the following structures need to be packed 1
#pragma pack( push, 1 )

typedef unsigned char byte;
typedef unsigned short word;

// File header
typedef struct MS3DHeader
{
	char ID[10];
	int version;
}MS3DHEADER;

// Vertex information
typedef struct MS3DVertex
{
	byte flags;
	float vertex[3];
	char boneID;
	byte refCount;
}MS3DVERTEX;

// Triangle information
typedef struct MS3DTriangle
{
	word flags;
	word vertexIndices[3];
	float vertexNormals[3][3];
	float s[3];
	float t[3];
	byte smoothingGroup;
	byte groupIndex;
}MS3DTRIANGLE;

// Material information
typedef struct MS3DMaterial
{
    char name[32];
    float ambient[4];
    float diffuse[4];
    float specular[4];
    float emissive[4];
    float shininess;	// 0.0f - 128.0f
    float transparency;	// 0.0f - 1.0f
    byte mode;	// 0, 1, 2 is unused now
    char texture[128];
    char alphamap[128];
}MS3DMATERIAL;

//	Joint information
typedef struct MS3DJoint
{
	byte flags;
	char name[32];
	char parentName[32];
	float rotation[3];
	float translation[3];
	word numRotationKeyframes;
	word numTranslationKeyframes;
} MS3DJOINT;

// Keyframe data
typedef struct MS3DKeyframe
{
	float time;
	float parameter[3];
}MS3DKEYFRAME;

typedef struct JointNameListRec
{
	int jointIndex;
	const char *pName;
}JOINTNAMELISTREC;

#pragma pack( pop )

//Load the model data into the private variables.
BOOL Model_LoadModelData(MODEL * pM, const char *filename );

#endif // _MILKSHAPEMODEL_H_
