// Timer.c

#include "Project.h"
#include <assert.h>
#include "Timer.h"

#include <mmsystem.h>

TIMER * Timer_GetNewTimer(void)
{
	TIMER * tim = malloc(sizeof(TIMER));
	Timer_Timer(tim);
	return tim;
}

void Timer_Timer(TIMER * tim)
{
	if ( QueryPerformanceFrequency(( long long* )&tim->frequency ) == FALSE )
	{
		// No performance counter available
		tim->usePerformanceCounter = FALSE;

		tim->mmTimerStart = timeGetTime();
	}
	else
	{
		// Performance counter is available, use it instead of the multimedia timer
		tim->usePerformanceCounter = TRUE;

		// Get the current time and store it in pcTimerStart
		QueryPerformanceCounter(( long long* )&tim->pcTimerStart );

		// Calculate the timer resolution using the timer frequency
		tim->resolution = ( float )( 1.0/( double )tim->frequency )*1000.0f;
	}

	tim->pauseCount = 0;
	tim->pauseTime = 0;
}

void Timer_Reset(TIMER * tim)
{
	if ( tim->usePerformanceCounter == TRUE )
		QueryPerformanceCounter(( long long* )&tim->pcTimerStart );
	else
		tim->mmTimerStart = timeGetTime();

	tim->pauseTime = Timer_GetTime(tim);
}

double Timer_GetTime(TIMER * tim)
{
	if ( tim->pauseCount > 0 )
		return tim->pauseTime;

	long long timeElapsed;

	if ( tim->usePerformanceCounter == TRUE )
	{
		QueryPerformanceCounter(( long long* )&timeElapsed );
		timeElapsed -= tim->pcTimerStart;
		return timeElapsed * tim->resolution;
	}
	else
	{
		timeElapsed = timeGetTime() - tim->mmTimerStart;
		return ( double )timeElapsed;
	}
}

void Timer_Pause(TIMER * tim)
{
	if ( tim->pauseCount == 0 )
		tim->pauseTime = Timer_GetTime(tim);

	tim->pauseCount++;
}

void Timer_Unpause(TIMER * tim)
{
	assert( tim->pauseCount > 0 );
	tim->pauseCount--;

	if ( tim->pauseCount == 0 )
	{
		if ( tim->usePerformanceCounter == TRUE )
		{
			long long time;
			QueryPerformanceCounter(( long long* )&time );
			tim->pcTimerStart = time - ( long long )( tim->pauseTime/tim->resolution );
		}
		else
		{
			tim->mmTimerStart = ( unsigned int)( timeGetTime() - tim->pauseTime );
		}
	}
}


