// Vector.c

#include <math.h>
#include "Vector.h"

void Vector_Reset(float * this)
{
	this[0] = this[1] = this[2] = 0.0f;
	this[3] = 1.0f;
}

void Vector_Add(float * this, const float * vector )
{
	this[0] += vector[0];
	this[1] += vector[1];
	this[2] += vector[2];
	this[3] += vector[3];
}

void Vector_Sub(float * this, const float * vector )
{
	this[0] -= vector[0];
	this[1] -= vector[1];
	this[2] -= vector[2];
	this[3] -= vector[3];
}

void Vector_Min(float * this, const float * vector)
{
  	this[0] = this[0] < vector[0] ? this[0] : vector[0];
	this[1] = this[1] < vector[1] ? this[1] : vector[1];
  	this[2] = this[2] < vector[2] ? this[2] : vector[2];
}

void Vector_Max(float * this, const float * vector)
{
  	this[0] = this[0] > vector[0] ? this[0] : vector[0];
	this[1] = this[1] > vector[1] ? this[1] : vector[1];
  	this[2] = this[2] > vector[2] ? this[2] : vector[2];
}

void Vector_Lerp(float * this, const float * vector, float s)
{
	this[0] = s * (vector[0] - this[0]);
	this[1] = s * (vector[1] - this[1]);
	this[2] = s * (vector[2] - this[2]);
}

void Vector_Normalize(float * this)
{
	float len = Vector_Length( this );

	this[0] /= len;
	this[1] /= len;
	this[2] /= len;
}

float Vector_Length(float * this)
{
	return ( float )sqrt( this[0] * this[0] + this[1] * this[1] + this[2] * this[2] );
}

float Vector_LengthSq(float * this)
{
	return ( float ) ( this[0] * this[0] + this[1] * this[1] + this[2] * this[2] );
}

void Vector_Set(float * this, const float *values )
{
	this[0] = values[0];
	this[1] = values[1];
	this[2] = values[2];
}

void Vector_Vector(float * this, const float *values )
{
	Vector_Set( this, values );
	this[3] = 1;
}

void Vector_Transform(float * this, const float * mat )
{
	double vector[4];

	vector[0] = this[0] * mat[0] + this[1] * mat[4] + this[2] * mat[8] + mat[12];
	vector[1] = this[0] * mat[1] + this[1] * mat[5] + this[2] * mat[9] + mat[13];
	vector[2] = this[0] * mat[2] + this[1] * mat[6] + this[2] * mat[10] + mat[14];
	vector[3] = this[0] * mat[3] + this[1] * mat[7] + this[2] * mat[11] + mat[15];

	this[0] = ( float )( vector[0] );
	this[1] = ( float )( vector[1] );
	this[2] = ( float )( vector[2] );
	this[3] = ( float )( vector[3] );
}

void Vector_Transform3(float * this, const float * mat )
{
	double vector[3];

	vector[0] = this[0] * mat[0] + this[1] * mat[4] + this[2] * mat[8];
	vector[1] = this[0] * mat[1] + this[1] * mat[5] + this[2] * mat[9];
	vector[2] = this[0] * mat[2] + this[1] * mat[6] + this[2] * mat[10];

	this[0] = ( float )( vector[0] );
	this[1] = ( float )( vector[1] );
	this[2] = ( float )( vector[2] );
	this[3] = 1.0f;
}

